"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TopicSubscriptionResource = exports.SQSResource = exports.SNSResource = exports.generateARN = void 0;
function generateARN(service, region, accountID, resourceName) {
    return 'arn:aws:' + service + ':' + region + ':' + accountID + ':' + resourceName;
}
exports.generateARN = generateARN;
class Resource {
    constructor() { }
    toString() {
        return 'Blank Resource';
    }
    async ensureAndCreate(client) {
        throw new Error('Attempted to create blank resource! Use SNSResource, SQSResource or TopicSubscriptionResource');
    }
}
class SNSResource extends Resource {
    constructor(topicName, region) {
        super();
        this.topicName = topicName;
        this.region = region;
    }
    toString() {
        return 'sns.' + this.topicName + '.' + this.region;
    }
    async ensureAndCreate(client) {
        console.log('Creating SNSResource: ' + this.topicName);
        await client.snsClient
            .createTopic({
            Name: this.topicName,
        })
            .promise();
    }
}
exports.SNSResource = SNSResource;
class SQSResource extends Resource {
    constructor(queueName, region, policyType) {
        super();
        this.queueName = queueName;
        this.region = region;
        this.policyType = policyType;
    }
    toString() {
        return 'sqs.' + this.queueName + '.' + this.region;
    }
    async ensureAndCreate(client) {
        console.log('Creating SQSResource: ' + this.queueName);
        let attributes = {};
        if (this.policyType == 'all-sns') {
            let accountID = await client.getAccountID();
            let newARN = generateARN('sqs', this.region, accountID, this.queueName);
            attributes['Policy'] = JSON.stringify({
                Version: '2012-10-17',
                Id: 'Policy1645582105705',
                Statement: [
                    {
                        Sid: 'Stmt1645582011402',
                        Effect: 'Allow',
                        Principal: {
                            AWS: 'arn:aws:iam::' + accountID + ':root',
                            Service: 'sns.amazonaws.com',
                        },
                        Action: 'SQS:*',
                        Resource: newARN,
                    },
                ],
            });
        }
        await client.sqsClient
            .createQueue({
            QueueName: this.queueName,
            Attributes: attributes,
        })
            .promise();
    }
}
exports.SQSResource = SQSResource;
class TopicSubscriptionResource extends Resource {
    constructor(sns, sqs, filterPolicy) {
        super();
        this.sns = sns;
        this.sqs = sqs;
        this.filterPolicy = filterPolicy;
    }
    toString() {
        return 'subscription[' + this.sqs.toString() + ',' + this.sns.toString() + ',' + JSON.stringify(this.filterPolicy) + ']';
    }
    async ensureAndCreate(client) {
        console.log('Creating TopicSubscriptionResource: {topic: ' + this.sns.topicName + ', queue: ' + this.sqs.queueName + ', policy: ' + JSON.stringify(this.filterPolicy) + '}');
        let attributes = {};
        if (this.filterPolicy != null) {
            attributes['FilterPolicy'] = JSON.stringify(this.filterPolicy);
        }
        let accountID = await client.getAccountID();
        let snsARN = generateARN('sns', this.sns.region, accountID, this.sns.topicName);
        let sqsARN = generateARN('sqs', this.sqs.region, accountID, this.sqs.queueName);
        await client.snsClient
            .subscribe({
            TopicArn: snsARN,
            Protocol: 'sqs',
            Endpoint: sqsARN,
            Attributes: attributes,
        })
            .promise();
    }
}
exports.TopicSubscriptionResource = TopicSubscriptionResource;
